local events = {}

-- PLACE EVENT HANDLERS HERE, NAMED AS THE EVENT THEY SHOULD HANDLE

--- Called for each combat log event.
function events:COMBAT_LOG_EVENT_UNFILTERED(timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, destName, destFlags, destFlags2)
  if (event == "PARTY_KILL") then
    Tally.record_mob_kill(Tally.ParseGUID(destGUID))
  end
end

--- Called for each mouseover unit event
function events:UPDATE_MOUSEOVER_UNIT(...)
  if (UnitGUID("mouseover")) then
	Tally.store_data("mouseover")
	Tally.update_tooltip("mouseover")
  end
end

function events:PLAYER_TARGET_CHANGED(arg1)
  if (UnitGUID("target")) then
    Tally.store_data("target")
	Tally.update_tooltip("target")
  end
end

--- Called when the loot window opens
function events:LOOT_OPENED(...)
  Tally.parseLootEvent()
end

--- Called when there's an emote from an NPC
-- @param msg The message emoted by the NPC. Localised. Contains a %s for where to put the mob name.
-- @param senderName The NAME (not the ID or similar useful item) of the NPC.
function events:CHAT_MSG_MONSTER_EMOTE(msg, senderName, ...)
  -- TODO: Localise the run string
  if msg:find("attempts to run away in fear!") then
    Tally.record_mob_runs_by_name(senderName)
  end
end

-- END OF EVENT HANDLER AREA

--- Register the events associated with events above.
function Tally.RegisterEvents(self)
  for func, _ in pairs(events) do
    self:RegisterEvent(func)
  end
end

--- Set up ACE3 localisation
function Tally.InitialiseLocalisation()
  Tally.L = LibStub("AceLocale-3.0"):GetLocale("Tally")
end

--- The addon has loaded.  Deal with anything we need to cover at load time.
function TallyLoadHandler(self)
  Tally.InitialiseLocalisation()
  print (Tally.L["WELCOME"])
  Tally.RegisterEvents(self)
  
  -- initialise the tooltip we are drawing to.
  -- Tally.initialise_tooltip_frame()
end

--- An event has occurred and we registered an interest in it.  Call our event handler for that event.
function TallyEventHandler(self, event, ...)
  events[event](self, ...)
end

--- Stores data on a unitId to the database
function Tally.store_data(unitId)
  local NPCid, mobType = Tally.ParseGUID(UnitGUID(unitId))
  -- Only store if it is a NPC
  if (mobType == "NPC") then
    local name = UnitName(unitId)
    local location = GetRealZoneText()
    local level = UnitLevel(unitId)
    Tally.record_mob(NPCid, name, location, level)
  end
end -- function Tally.store_data

--- Updates the tooltip frame
function Tally.update_tooltip(unitId)
	local unitName = UnitName(unitId)
	local unitLevel = UnitLevel(unitId)
	local unitClass, englishClass, classNo = UnitClass(unitId)
	-- ToDo: localise these values or work out why unitClass is not working properly
	if (classNo == 0) then unitClass = "None" end
	if (classNo == 1) then unitClass = "Warrior" end
	if (classNo == 2) then unitClass = "Paladin" end
	if (classNo == 3) then unitClass = "Hunter" end
	if (classNo == 4) then unitClass = "Rogue" end
	if (classNo == 5) then unitClass = "Priest" end
	if (classNo == 6) then unitClass = "DeathKnight" end
	if (classNo == 7) then unitClass = "Shaman" end
	if (classNo == 8) then unitClass = "Mage" end
	if (classNo == 9) then unitClass = "Warlock" end
	if (classNo == 10) then unitClass = "Monk" end
	if (classNo == 11) then unitClass = "Druid" end
	
	local unitCreatureType = UnitCreatureType(unitId)
	
	Tally_MobLevel:SetText("[" .. unitLevel .. "] ")
	Tally_MobLevel:SetTextColor(UnitSelectionColor(unitId))
	Tally_MobName:SetText(unitName)
	Tally_MobName:SetTextColor(GameTooltip_UnitColor("mouseover"))
	Tally_ClassInfo:SetText(unitClass .. ", " .. unitCreatureType)
	
	Tally_HealthLabel:SetTextColor(1, 1, 0, 1)
	Tally_HealthLabel:SetText("Health")
	
	local currentHealth = UnitHealth(unitId)
	local maxHealth = UnitHealthMax(unitId)
	
	Tally_HealthText:SetText(" " .. currentHealth .. " / " .. maxHealth)
	
	local powerType = UnitPowerType(unitId)
	if (powerType == 0) then
		Tally_ManaLabel:SetText("Mana")
	end
	if powerType == 1 then
		Tally_ManaLabel:SetText("Rage")
	end
	if powerType == 2 then
		Tally_ManaLabel:SetText("Focus")
	end
	if powerType == 3 then
		Tally_ManaLabel:SetText("Energy")
	end
	if powerType == 6 then
		Tally_ManaLabel:SetText("Runic Power")
	end
	Tally_ManaLabel:SetTextColor(1, 1, 0, 1)
	
	local currentPower = UnitPower(unitId)
	local maxPower = UnitPowerMax(unitId)
	
	Tally_ManaLevel:SetText(" " .. currentPower .. " / " .. maxPower)
	
	-- kills, loots and runs - only show if we are looking at a npc
	local unitGuid = UnitGUID(unitId)
	local NPCid, mobType = Tally.ParseGUID(UnitGUID(unitId))
	local kills, loots, runs = Tally.get(NPCid, unitLevel)
	
	if NPCid == "NPC" then
		if runs then
			Tally_Runs:SetTextColor(1, 0, 0, 1)
			Tally_Runs:SetText("*runs*")
		else
			Tally_Runs:SetText("")
		end -- if runs
		if kills then
		end -- if kills
	end -- if
	
	Tally_TooltipFrame:ClearAllPoints()
	Tally_TooltipFrame:SetPoint("BOTTOMLEFT", Tally_TooltipAnchor, "TOPLEFT")
	Tally_TooltipFrame:Show()
end -- function Tally.update_tooltip

function Tally.get_extra_tooltip_data()
	-- We can have up to 4 lines of extra data
	-- Apart from faction and skinnable details
	local numLines = GameTooltip:NumLines()
end

--- Hides the tooltip frame
function Tally.hide_tooltip()
	Tally_TooltipFrame:Hide()
end -- function Tally.hide_tooltip

--- Initialises the tooltip frame
function Tally.initialise_tooltip_frame()
	Tally_TooltipFrame:SetBackdrop( { bgFile = "Interface/Tooltips/UI-Tooltip-Background",
									   edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
									   tile = true, tileSize = 24, edgeSize = 12,
									   insets = { left=4, right=4, top=4, bottom=4 } } )
	Tally_TooltipFrame:SetBackdropColor(0, 0, 0, 0.8)
	Tally_TooltipFrame:SetBackdropBorderBolor(0.5, 0, 0, 1)
	
	Tally_TooltipFrame:SetClampedToScreen(true)
	
	Tally.update_tooltip()
end
